% program calculates the electric potential in unbiased pn homojunction
% version 15.08.13, 
clear  % clears variables
clear global  % clears global variables
format long e
% initial constants
I=sqrt(-1); % remember not to overwrite pi or i !
pi=3.141592653589793e+000;
c=2.998e+2; % speed of light in free space [mi/ps]
q=1.602e-19; % elemental charge [C]
h_s=6.62606885e-34; % Planck's constant [J*ps]
kB=1.3807e-23; % Boltzman's constant [J/K]
T=300.; % temperature [K]
kT=kB*T/q; % kT [eV]
UT=kB*T/q; % UT thermal potential [V]
eps0=8.854e-18; % vacum permittivity [F/mi]
m0=9.11e-31; % electron mass [kg]

% input semiconductor parameters (GaAs) 
mc=0.067; % relative electron mass 
mv=0.37736; % relative hole mass 
%(electrons) effective density of states [1/mi3]:
Nc=2*sqrt((2*pi*mc*m0*kB*T/(h_s*h_s))^3)*1.e-18 
%(holes) effective density of states [1/mi3]:
Nv=2*sqrt((2*pi*mv*m0*kB*T/(h_s*h_s))^3)*1.e-18 

Wg=1.424; % bandgap  [eV]
X=4.07; % affinity [eV]
Wc=-4.07; % conduction band edge [eV] measured with respect to vacuum level
Wv=-5.494; % valence band edge [eV] measured with respect to vacuum level
eps=13.18; % relative dielectric constant

% n side
ND=1e+5; % donor concentration [1/mi3]
thick_n=10.0; % n side thickness [mi]
% Fermi level at no bias measured with respect to vac level on n side [eV]:
WFeq_n=Wc+kT*log(ND/Nc) 

% p side (the unchanged params are not redefined)
NA=1e+4; % acceptor concentration [1/mi3]
thick_p=10.0; % p side thickness [mi]
% Fermi level at no bias measured with respect to vac level on n side [eV]:
WFeq_p=Wv-kT*log(NA/Nv)
% Barrier potential (can be calculated directly since energy is in eV):
VBarrier=WFeq_n-WFeq_p 

Up=(-X-Wg-WFeq_n);
Un=(X+WFeq_n);

% calculation parameters
thick=(thick_n+thick_p); % device length 
N_points=1000; % number of points on L
dx=thick/(N_points); % step dx [Li]

% vector initialisation
EQ=zeros(N_points,1);
Vi=zeros(N_points,1);
left_diag=zeros(N_points,1);
right_diag=zeros(N_points,1);
KN=zeros(N_points,1);
ue=ones(N_points+2,1);

% initial voltage distribution
for i=1:N_points/2
    Vi(i)=0.0;
    Vi(i+N_points/2)=-VBarrier;
end
ue(2:N_points+1)=Vi;
ue(1)=0;
ue(N_points+2)=-VBarrier;

% setting up vectors for equations
for i=1:N_points/2
    KN(i)=ND;
    KN(i+N_points/2)=-NA;
end
w=log(abs(KN(1)))


% setting off diagonals of Jacobian
for i=1:N_points
    left_diag(i)=-1.0/(dx*dx);
    right_diag(i)=-1.0/(dx*dx);
end 

for j=1:10 %main loop start
% setting main diagonal of Jacobian
 main_diag=2.0/(dx*dx)+(1/UT)*...
(Nv*exp((Up-Vi)/UT)+Nc*exp((Un+Vi)/UT))*q/(eps*eps0); 
 Jacobian=spdiags([left_diag main_diag right_diag],-1:1,N_points,N_points);

% calculating the function value at Vi
 for i=2:N_points+1
 EQ(i-1)=-(q/(eps*eps0))*(Nv*exp((Up-ue(i))/UT)-Nc*exp((Un+ue(i))/UT)+...
 KN(i-1))+(2.0*ue(i)-ue(i-1)-ue(i+1))/(dx*dx);
 end 

Right_side=-EQ+Jacobian*Vi;
Vi=Jacobian\Right_side;
res=ue(2:N_points+1)-Vi;
res'*res
ue(2:N_points+1)=Vi;

end %main loop stop

z_pos=dx*(1:N_points)';
Vacuum=-Vi;
Ec=Wc+Vacuum;
Ev=Wv+Vacuum;
Fermi=WFeq_n*ones(N_points,1);

hold on
plot(z_pos,Vi,'Linewidth',2,'Color',[0. 0. 0.])

plot(z_pos,Fermi,'--','Linewidth',2,'Color',[0. 0. 0.])

plot(z_pos,Vacuum,'Linewidth',2,'Color',[0.4 0.4 0.4])

plot(z_pos,Ec,'Linewidth',2,'Color',[0.4 0.4 0.4])

plot(z_pos,Ev,'Linewidth',2,'Color',[0.4 0.4 0.4])

xlabel('x/micrometer','FontSize',24)
ylabel('Energy/eV','FontSize',24) 

