% program calculates the band structure of biased diode,Bernoulli
% version 23.08.13, with SRH recombination GaAs
clear  % clears variables
clear global  % clears global variables
format long e
%format short
% initial constants
I=sqrt(-1); % remember not to overwrite pi or i !
pi=3.141592653589793e+000;
c=2.998e+2; % speed of light in free space [mi/ps]
q=1.602e-19; % elemental charge [C]
h_s=6.62606885e-34; % Planck's constant [J*ps]
kB=1.3807e-23; % Boltzman's constant [J/K]
T=300.; % temperature [K]
kT=kB*T/q; % kT [eV]
UT=kB*T/q; % UT thermal potential [V]
eps0=8.854e-18; % vacum permittivity [F/mi]
m0=9.11e-31; % electron mass [kg]
miu_n=0.4; %electron mobility [mi^2/(V*ps)]
miu_p=0.04; %hole mobility [mi^2/(V*ps)]

% input diode parameters (GaAs)
mc_n=0.067; % relative electron mass 
mv_n=0.37736; % relative hole mass 
Wg_n=1.424; % bandgap  [eV]
X_n=4.07; % affinity [eV]
Wc_n=-4.07; % bottom of conduction band [eV] measured with respect to vacuum level
Wv_n=-5.494; % top of valence band [eV] measured with respect to vacuum level
eps_n=13.18; % relative dielectric constant

% n side first
ND=1e+4; % effective dopant concentration [1/mi3]
thick_n=10.0; % thickness [mi]

% p side 
NA=ND; % effective dopant concentration [1/mi3] 
thick_p=10.0; % thickness [mi]

% calculating basic parameters for n side and p side:
% n side:
Nc_n=2*sqrt((2*pi*mc_n*m0*kB*T/(h_s*h_s))^3)*1.e-18 %electron effective density of states [1/mi3]
Nv_n=2*sqrt((2*pi*mv_n*m0*kB*T/(h_s*h_s))^3)*1.e-18 %hole effective density of states [1/mi3]
ni_n=sqrt(Nc_n*Nv_n)*exp(-Wg_n/(2.0*kT)) % intrinsic carrier concentration [1/mi3]
Wi_n=(Wc_n+Wv_n)/2.0-3.0*kT*log(mc_n/mv_n)/4.0 % intrinsic Fermi measured with respect to vac level on n side [eV]
WFeq_n=Wi_n+kT*log(abs(ND)/ni_n) % Fermi level at no bias measured with respect to vac level on n side [eV]

% p side:
WFeq_p=Wi_n-kT*log(abs(NA)/ni_n) % Fermi level at no bias measured with respect to vac level on p side [eV]
VBarrier=WFeq_n-WFeq_p % Barrier potential (can be calculated directly since energy is expressed in eV)

% recombination coefficient eps*Bim/(q*miu)
tau=10; %SRH lifetime [ps]
B_n=eps_n*eps0/(tau*ni_n*q*miu_n);
B_p=eps_n*eps0/(tau*ni_n*q*miu_p);

% normalising relevant parameters
Nc_n=Nc_n/ni_n
Nv_n=Nv_n/ni_n
ND=ND/ni_n
NA=NA/ni_n

Wg_n=Wg_n/UT
Li=sqrt(eps_n*eps0*UT/(q*ni_n)) %Debye length
VBarrier=VBarrier/UT

% calculation parameters
thick=(thick_n+thick_p)/Li; % device length [Li-Debye length]
N_points=1000; % number of points on L
dx=thick/(N_points); % step dz [Li]
x_pos=dx*(1:N_points)';

% vector initialisation
EQ=zeros(N_points,1);
neq=zeros(N_points,1);
peq=zeros(N_points,1);
ui=zeros(N_points,1);
vi=zeros(N_points,1);
wi=zeros(N_points,1);
a1=zeros(N_points,1);
a2=zeros(N_points,1);
a3=zeros(N_points,1);
b1=zeros(N_points,1);
b2=zeros(N_points,1);
b3=zeros(N_points,1);
A1=zeros(N_points,1);
A2=zeros(N_points,1);
A3=zeros(N_points,1);
A4=zeros(N_points,1);
AP1=zeros(N_points,1);
AP2=zeros(N_points,1);
B1=zeros(N_points,1);
B2=zeros(N_points,1);
B3=zeros(N_points,1);
B4=zeros(N_points,1);
BP1=zeros(N_points,1);
BP2=zeros(N_points,1);
E1=zeros(N_points,1);
E2=zeros(N_points,1);
DEN=zeros(N_points,1);
NUM1=zeros(N_points,1);
NUM2=zeros(N_points,1);
left_diag=zeros(N_points,1);
main_diag=zeros(N_points,1);
right_diag=zeros(N_points,1);
KN=zeros(N_points,1);
ue=ones(N_points+2,1);
ve=ones(N_points+2,1);
we=ones(N_points+2,1);

% setting doping profile
for i=1:N_points/2
    KN(i)=ND;
    KN(i+N_points/2)=-NA;    
end
w=log(abs(KN(1))) %boundary condition for Fermi levels


% initial voltage and quasi-Fermi level distribution
for i=1:N_points/2
    ui(i)=0.0;
    ui(i+N_points/2)=-(VBarrier);
    vi(i)=-w;
    vi(i+N_points/2)=-w;   
    wi(i)=-w;
    wi(i+N_points/2)=-w;    
end

ue(2:N_points+1)=ui;
ve(2:N_points+1)=vi;
we(2:N_points+1)=wi;

% bias voltage
VBias=0.20; %bias voltage [V]
dVBias=0.2; %bias voltage step
%normalisation of bias voltage and of the bias voltage step
VBias=VBias/UT; 
dVBias=dVBias/UT; 

for jbias=1:1 %external loop stepping up bias voltage
VBias*UT
ue(1)=0;
ue(N_points+2)=-(VBarrier-VBias);
ve(1)=-w;
ve(N_points+2)=-(w-VBias);
we(1)=-w;
we(N_points+2)=-(w-VBias);



for j=1:20 %main loop start
j
% setting auxiliary vectors
for i=1:N_points
a1(i)=ue(i+1)-ve(i+1)-ue(i+2)+ve(i+2);
a2(i)=ue(i+1)-ve(i+1)-ue(i)+ve(i);
b1(i)=we(i+1)-ue(i+1)-we(i+2)+ue(i+2);
b2(i)=we(i+1)-ue(i+1)-we(i)+ue(i);
A3(i)=(ve(i+2)-ve(i+1))/(dx*dx);
A4(i)=(ve(i+1)-ve(i))/(dx*dx);
B3(i)=(we(i+2)-we(i+1))/(dx*dx);
B4(i)=(we(i+1)-we(i))/(dx*dx);
end

for i=1:N_points
a3(i)=ui(i)-vi(i);
b3(i)=wi(i)-ui(i);
A1(i)=Bern(a1(i));
A2(i)=Bern(a2(i));
AP1(i)=Bernp(a1(i));
AP2(i)=Bernp(a2(i));
B1(i)=Bern(b1(i));
B2(i)=Bern(b2(i));
BP1(i)=Bernp(b1(i));
BP2(i)=Bernp(b2(i));
E1(i)=exp(a3(i));
E2(i)=exp(b3(i));
end

DEN=E1+E2+2;
NUM1=E2-1./E1;
NUM2=E1-1./E2;

% setting off diagonals of Jacobian for dEQudu
for i=1:N_points
    left_diag(i)=1.0/(dx*dx);
    right_diag(i)=1.0/(dx*dx);
end

% setting dEQudu, dEQudv, dEQudw and EQu
main_diag=-2.0/(dx*dx)-E1-E2; 
dEQudu=spdiags([left_diag main_diag right_diag], -1:1, N_points, N_points);
dEQudv=spdiags([E1],0,N_points, N_points);
dEQudw=spdiags([E2],0,N_points, N_points);
for i=2:N_points+1
    EQu(i-1)=-(2.0*ue(i)-ue(i-1)-ue(i+1))/(dx*dx)-exp(ue(i)-ve(i))+exp(we(i)-ue(i))+KN(i-1);
end 

% setting dEQvdu, dEQvdv, dEQvdw and EQv
left_diag=leftshift(AP2.*A4); 
main_diag=AP1.*A3-AP2.*A4+B_n*((-E2+1./E1).*DEN-NUM1.*(E1+E2))./(DEN.*DEN); 
right_diag=rigshift(-AP1.*A3); 
dEQvdu=spdiags([left_diag main_diag right_diag], -1:1, N_points, N_points);

left_diag=leftshift(-AP2.*A4+A2/(dx*dx)); 
main_diag=-AP1.*A3+AP2.*A4-A1/(dx*dx)-A2/(dx*dx)+B_n*((-DEN./E1)+NUM1.*E1)./(DEN.*DEN);
right_diag=rigshift(AP1.*A3+A1/(dx*dx)); 
dEQvdv=spdiags([left_diag main_diag right_diag], -1:1, N_points, N_points);

dEQvdw=spdiags([B_n*(E2.*DEN-NUM1.*(E1+E2))./(DEN.*DEN)], 0, N_points, N_points);

EQv=A1.*A3-A2.*A4+B_n*(E2-1./E1)./DEN;

% setting dEQwdu, dEQwdv, dEQwdw and EQw
left_diag=leftshift(-BP2.*B4); 
main_diag=-BP1.*B3+BP2.*B4-B_p*((E1-1./E2).*DEN-NUM2.*(E1+E2))./(DEN.*DEN);
right_diag=rigshift(BP1.*B3); 
dEQwdu=spdiags([left_diag main_diag right_diag], -1:1, N_points, N_points);

left_diag=leftshift(BP2.*B4+B2/(dx*dx));  
main_diag=BP1.*B3-BP2.*B4-B1/(dx*dx)-B2/(dx*dx)-B_p*((DEN./E2)-NUM2.*E2)./(DEN.*DEN);
right_diag=rigshift(-BP1.*B3+B1/(dx*dx)); 
dEQwdw=spdiags([left_diag main_diag right_diag], -1:1, N_points, N_points);

dEQwdv=spdiags([-B_p*((DEN.*E1)+NUM2.*E1)./(DEN.*DEN)], 0, N_points, N_points);

EQw=B1.*B3-B2.*B4-B_p*(E1-1./E2)./DEN;

% setting the Jacobian and the vector
Jacobian=[dEQudu dEQudv dEQudw;dEQvdu dEQvdv dEQvdw;dEQwdu dEQwdv dEQwdw];
Vi=[ui;vi;wi];
dVi=Vi;
EQ=[EQu';EQv;EQw];

dVi=-Jacobian\EQ;
Vi=Vi+dVi;
ui=Vi(1:N_points);vi=Vi(N_points+1:2*N_points);wi=Vi(2*N_points+1:3*N_points);
ue(2:N_points+1)=ui;ve(2:N_points+1)=vi;we(2:N_points+1)=wi;
dVi'*dVi
end %main loop stop
VBias=VBias+dVBias;
end %external loop stop

%plot(x_pos*Li,ui*UT,x_pos*Li,-ui*UT,x_pos*Li,-ui*UT-Wg_n*UT/2,x_pos*Li,-ui*UT+Wg_n*UT/2,x_pos*Li,-vi*UT,x_pos*Li,-wi*UT)
hold on
plot(x_pos*Li,ui*UT,'Linewidth',2,'Color',[0. 0. 0.])

plot(x_pos*Li,-ui*UT,'Linewidth',2,'Color',[0.4 0.4 0.4])

plot(x_pos*Li,-vi*UT,'--','Linewidth',2,'Color',[0. 0. 0.])

plot(x_pos*Li,-wi*UT,'--','Linewidth',2,'Color',[0.4 0.4 0.4])

plot(x_pos*Li,-ui*UT+Wg_n*UT/2,'Linewidth',2,'Color',[0.4 0.4 0.4])

plot(x_pos*Li,-ui*UT-Wg_n*UT/2,'Linewidth',2,'Color',[0.4 0.4 0.4])

xlabel('x/micrometer','FontSize',24)
ylabel('Energy/eV','FontSize',24) 
